<?php
// vim: set ts=4 sw=4 sts=4 et:

/**
 * Copyright (c) 2018-present gotoBilling, Inc. All rights reserved.
 */

namespace XLite\Module\Omnifund\Payments\Model\Payment;

use XLite\Core\Config;
use XLite\Core\Converter;

/**
 * Payment method model
 */
class Method extends \XLite\Model\Payment\Method implements \XLite\Base\IDecorator
{
    /**
     * Get message why we can't switch payment method
     *
     * @return string
     */
    public function getNotSwitchableReason()
    {
        $message   = parent::getNotSwitchableReason();
        $processor = $this->getProcessor();

        if ($processor
            && 'OmniPayment' === $this->getServiceName()
        ) {
            if ($this->getSetting('merchant_id')
                && $this->getSetting('client_id')
                && !Config::getInstance()->Security->customer_security
            ) {
                $message = static::t(
                    'The "OmniFund Payments" module requires HTTPS to be properly set up for your store.',
                    [
                        'url' => Converter::buildURL('https_settings'),
                    ]
                );
            } else {
                $message = static::t('The "OmniFund Payments" module is not configured.  Please configure it before using.');
            }
        }

        return $message;
    }
}
