<?php
/**
 * Copyright (c) 2018-present gotoBilling, Inc. All rights reserved.
 */

namespace XLite\Module\Omnifund\Payments\Model\Payment\Processor;

class OmniPayment extends \XLite\Module\Omnifund\Payments\Model\Payment\Processor\OmnifundGateway
{

    /**
     * Get allowed backend transactions
     *
     * @return string Status code
     */
    public function getAllowedTransactions()
    {
        return array();
    }

    /**
     * Get return type
     *
     * @return string
     */
    public function getReturnType()
    {
        return self::RETURN_TYPE_HTTP_REDIRECT;
    }

		public function getSettingsWidget()
    {
    		if (version_compare(\XLite::getInstance()->getVersion(), '5.3', '>=')) 
            return 'modules/OmniFund/Payments/OmniFundConfig.twig';
				else
            return 'modules/OmniFund/Payments/OmniFundConfig.tpl';
    }


    /**
     * Check - payment method is configured or not
     *
     * @param \XLite\Model\Payment\Method $method Payment method
     *
     * @return boolean
     */
    public function isConfigured(\XLite\Model\Payment\Method $method)
    {
        return parent::isConfigured($method)
            && $method->getSetting('accesskey')
            && $method->getSetting('accesskeysecret');
    }

    /**
     * Check - is method available for checkout
     *
     * @param \XLite\Model\Payment\Method $method Payment method
     *
     * @return boolean
     */
    public function isAvailable(\XLite\Model\Payment\Method $method)
    {
        return true;
    }

    /**
     * Check - is Method applicable for Order (depends on method requirements)
     *
     * @param \XLite\Model\Order $order
     * @param \XLite\Model\Payment\Method $method Payment method
     *
     * @return boolean
     */
    public function isApplicable(\XLite\Model\Order $order, \XLite\Model\Payment\Method $method)
    {
        return parent::isApplicable($order, $method)
            && $this->isAvailable($method);
    }

    /**
     * Check - payment method has enabled test mode or not
     *
     * @param \XLite\Model\Payment\Method $method Payment method
     *
     * @return boolean
     */
    public function isTestMode(\XLite\Model\Payment\Method $method)
    {
        return \XLite\View\FormField\Select\TestLiveMode::TEST == $method->getSetting('mode');
    }

    /**
     * Payment method has settings into Module settings section
     *
     * @return boolean
     */
    public function hasModuleSettings()
    {
        return true;
    }

    /**
     * Get payment method admin zone icon URL
     *
     * @param \XLite\Model\Payment\Method $method Payment method
     *
     * @return string
     */
    public function getAdminIconURL(\XLite\Model\Payment\Method $method)
    {
        return true;
    }

    /**
     * Define the fields for Transaction Data
     *
     * @return array
     */
    protected function defineSavedData()
    {
        $data = array(
            'order_number' => 'OrderNumber',
            'termination_code' => 'TerminationCode',
            'invoice_id' => 'InvoiceId',
            'status' => 'Status',
            'timestamp' => 'Timestamp',
            'amount' => 'Amount',
            'auth_code' => 'AuthCode',
            'termination_description' => 'TerminationDescription',
        );

        return $data;
    }

    /**
     * Process the payment and create transaction
     *
     * @return string $status Transaction Status
     */
    protected function doInitialPayment()
    {
        try {
		        $requestData = \XLite\Core\Request::getInstance()->getData();
				
						$this->OmniSetAccessKey( $this->getSetting('accesskey') );
						$this->OmniSetAccessKeySecret( $this->getSetting('accesskeysecret') );
						$this->OmniSetIpAddress( $this->getClientIP() );
						
						// Set Test Mode if necessary
						if (\XLite\View\FormField\Select\TestLiveMode::TEST == $this->getSetting('mode'))
							$this->OmniSetDebug( 'true' );
			 
						//$this->OmniSetCompany( $order_data['billing']['company'] );
			
						$this->OmniSetFirstName( $this->getProfile()->getBillingAddress()->getFirstname() );
						$this->OmniSetLastName( $this->getProfile()->getBillingAddress()->getLastname() );
						$this->OmniSetAddress1( $this->getProfile()->getBillingAddress()->getStreet() );
						//$this->getProfile()->getBillingAddress()->getAddress2()
						$this->OmniSetCity( $this->getProfile()->getBillingAddress()->getCity() );
						$this->OmniSetState( $this->getProfile()->getBillingAddress()->getState()->getCode() );
						$this->OmniSetZipCode( $this->getProfile()->getBillingAddress()->getZipcode() );
						$this->OmniSetCountry( $this->getProfile()->getBillingAddress()->getCountry()->getCode() );
						$this->OmniSetPhone( $this->getProfile()->getBillingAddress()->getPhone() );
						$this->OmniSetEmail( $this->getProfile()->getLogin() );
						
						/* 
							AS  Authorize Only 
							DS  Capture Only 
							ES  Authorize & Capture 
							CR  Credit/Refund 
							VO  Void 
							AV  AVS Check Only 
							OF  Offline (Force) 
							*/
							
						$this->OmniSetTransactionType( 'ES' );
						$this->OmniSetCcName( $requestData['omnifund_cardholder'] );	
						$this->OmniSetCcNumber( str_replace("-", "", $requestData['omnifund_cc_number']) );
						$this->OmniSetCcExpiration( sprintf('%02d', $requestData['omnifund_cc_exp_month']) . substr($requestData['omnifund_cc_exp_year'], -2) );
						$this->OmniSetCcVerification( $requestData['omnifund_cvv'] );
							
						$this->OmniSetAmount( round((double)($this->transaction->getValue()), 2) * 100 );
						
						//$this->OmniSetInvoiceId( '123456' );
						
						$this->OmniSetCustomerId( $this->getProfile()->getLogin() );
										
						$this->OmniProcess();
						
						if ($this->OmniGetStatus() == "G" ) {
							// Success
              $status = self::COMPLETED;
              
              // Create the backend transaction, and then save all info.
			        if (!$this->transaction->getInitialBackendTransaction()) {
			            $this->transaction->createBackendTransaction(\XLite\Model\Payment\BackendTransaction::TRAN_TYPE_CAPTURE);
			        }
			
			        $backendTransaction = $this->transaction->getInitialBackendTransaction();
			        if ($backendTransaction != null) {
			        	$backendTransaction->setStatus(\XLite\Model\Payment\Transaction::STATUS_SUCCESS);
		        	}
			
			        $this->setDetail('order_number', $this->OmniGetOrderNumber(), 'OrderNumber');
			        $this->setDetail('termination_code', $this->OmniGetTerminationCode(), 'TerminationCode');
			        $this->setDetail('invoice_id', $this->OmniGetInvoiceId(), 'InvoiceId');
			        $this->setDetail('status', $this->OmniGetStatus(), 'Status');
			        $this->setDetail('timestamp', $this->OmniGetTransactionDate() . " " . $this->OmniGetTransactionTime(), 'Timestamp');
			        $this->setDetail('amount', $this->OmniGetTransactionAmount(), 'Amount');
			        $this->setDetail('auth_code', $this->OmniGetAuthCode(), 'AuthCode');
			        $this->setDetail('termination_description', $this->OmniGetTerminationDescription(), 'TerminationDescription');
			        
			        if ($backendTransaction != null) {
				        $backendTransaction->update();
				      }
			
              $this->transaction->getOrder()->setPaymentStatus(\XLite\Model\Order\Status\Payment::STATUS_PAID);

			        $this->transaction->setType(\XLite\Model\Payment\BackendTransaction::TRAN_TYPE_CAPTURE);
			        $this->transaction->setStatus(\XLite\Model\Payment\Transaction::STATUS_SUCCESS);
			        $this->transaction->setNote(static::t('Payment completed successfully!'));
			        $this->transaction->registerTransactionInOrderHistory('Notification, OmniFund');

			        $this->transaction->update();

              \XLite\Core\TopMessage::getInstance()->addInfo("Payment succeeded!");
						} else {
							// Fail
		        	\XLite\Logger::logCustom("omnifund", 
		        													 "Charge Failed: Status: " . $this->OmniGetStatus() . "  Message: " . $this->OmniGetTerminationDescription(),
		        													 false);

              $this->transaction->setDataCell('status', static::t($this->OmniGetTerminationDescription()), null, static::FAILED);
              $this->transaction->setNote(static::t($this->OmniGetTerminationDescription()));
  	          $status = self::FAILED;
						}

        } catch (\Exception $e) {
	        	\XLite\Logger::logCustom("omnifund", 
	        													 'Error processing payment: ' . $e->getMessage(), 
	        													 false);

            $errorMessage = static::t('Error processing payment: ' . $e->getMessage());
            $this->transaction->setDataCell('status', $errorMessage, null, static::FAILED);
            $this->transaction->setNote($errorMessage);
            $status = self::FAILED;
        }

        return $status;
    }

    /**
     * Checkout Template Path
     *
     * $param \XLite\Model\Payment\Method $method
     *
     * @return string|null
     */
    public function getCheckoutTemplate(\XLite\Model\Payment\Method $method)
    {
    		if (version_compare(\XLite::getInstance()->getVersion(), '5.3', '>=')) 
            return 'modules/OmniFund/Payments/OmnifundCheckout.twig';
				else
            return 'modules/OmniFund/Payments/OmnifundCheckout.tpl';
    }

    /**
     * Input Template Path
     *
     * @return string|null
     */
    public function getInputTemplate()
    {
    		if (version_compare(\XLite::getInstance()->getVersion(), '5.3', '>=')) 
            return 'modules/OmniFund/Payments/OmnifundPayment.twig';
        else
            return 'modules/OmniFund/Payments/OmnifundPayment.tpl';
    }   

}
